/****************************************************************************
  
    PROGRAM: memgauge.c
  
    PURPOSE: Free memory gauge
  
    FUNCTIONS:
  
    WinMain() - calls initialization function, processes message loop
    InitApplication() - initializes window data and registers window
    InitInstance() - saves instance handle and creates main window
    MainWndProc() - processes messages
    About() - processes messages for "About" dialog box
  
    COMMENTS:
  
        Windows can have several copies of your application running at the
        same time.  The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.
  
****************************************************************************/
  
#include "windows.h"            /* required for all Windows applications */
#include "memgauge.h"           /* specific to this program              */
HPEN     hSolidPen;             /* pen handle         */
HBRUSH   hOldBrush;             /* old brush handle   */
HBRUSH   hMyBrush;              /* new brush handle   */

static long dwInstFreeMemK;

#define  TIMER_ID  1
#define  MY_WIDTH  320
#define  MY_HEIGHT 70
  
HANDLE hInst;                   /* current instance   */
  
/****************************************************************************
  
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
  
    PURPOSE: calls initialization function, processes message loop
  
    COMMENTS:
  
        Windows recognizes this function by name as the initial entry point
        for the program.  This function calls the application initialization
        routine, if no other instance of the program is running, and always
        calls the instance initialization routine.  It then executes a message
        retrieval and dispatch loop that is the top-level control structure
        for the remainder of execution.  The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application
        instance by returning the value passed by PostQuitMessage().
  
        If this function must abort before entering the message loop, it
        returns the conventional value NULL.
  
****************************************************************************/
  
int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                /* current instance             */
HANDLE hPrevInstance;            /* previous instance            */
LPSTR lpCmdLine;                 /* command line                 */
int nCmdShow;                    /* show-window type (open/icon) */
    {
    MSG msg;                     /* message                      */
  
    if (!hPrevInstance)          /* Other instances of app running?   */
        if (!InitApplication(hInstance)) /* Initialize shared things  */
            return (FALSE);      /* Exits if unable to initialize     */
  
    /* Perform initializations that apply to a specific instance      */
  
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);
  
    /* Acquire and dispatch messages until a WM_QUIT message is received. */
  
    while (GetMessage(&msg,    /* message structure              */
        NULL,          /* handle of window receiving the message */
        NULL,          /* lowest message to examine              */
        NULL))         /* highest message to examine             */
        {
        TranslateMessage(&msg); /* Translates virtual key codes           */
        DispatchMessage(&msg);  /* Dispatches message to window           */
        }
    return (msg.wParam);        /* Returns the value from PostQuitMessage */
    }
  
  
/****************************************************************************
  
    FUNCTION: InitApplication(HANDLE)
  
    PURPOSE: Initializes window data and registers window class
  
    COMMENTS:
  
        This function is called at initialization time only if no other
        instances of the application are running.  This function performs
        initialization tasks that can be done once for any number of running
        instances.
  
        In this case, we initialize a window class by filling out a data
        structure of type WNDCLASS and calling the Windows RegisterClass()
        function.  Since all instances of this application use the same window
        class, we only need to do this when the first instance is initialized.
  
  
****************************************************************************/
  
BOOL InitApplication(hInstance)
HANDLE hInstance;                  /* current instance */
    {
    WNDCLASS  wc;
  
    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */
  
    wc.style = NULL;                    /* Class style(s).                    */
    wc.lpfnWndProc = MainWndProc;       /* Function to retrieve messages for  */
                                        /* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance  = hInstance;          /* Application that owns the class.   */

    #if 0
    wc.hIcon      = LoadIcon(hInstance, "memgauge"); /* fixed icon */
    #endif

    wc.hIcon      = NULL;               /* This will cause the icon to be
                                           drawn, which allows updates while
                                           the window is iconized */

    wc.hCursor    = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName  = "MemGaugeMenu";  /* Name of menu resource in .RC file. */
    wc.lpszClassName = "MemGaugeWClass";/* Name used in call to CreateWindow. */
  
    /* Register the window class and return success/failure code. */
  
    return (RegisterClass(&wc));
  
    }
  
  
/****************************************************************************
  
    FUNCTION:  InitInstance(HANDLE, int)
  
    PURPOSE:  Saves instance handle and creates main window
  
    COMMENTS:
  
        This function is called at initialization time for every instance of
        this application.  This function performs initialization tasks that
        cannot be shared by multiple instances.
  
        In this case, we save the instance handle in a static variable and
        create and display the main program window.
  
****************************************************************************/
  
BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.       */
int             nCmdShow;           /* Param for first ShowWindow() call. */
    {
    HWND            hWnd;           /* Main window handle.                */

    char buffer[50];
  
    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */
  
    hInst = hInstance;
  
    dwInstFreeMemK = (GetFreeSpace(0) / 1024);

    if (GetWinFlags() & WF_LARGEFRAME || GetWinFlags() & WF_SMALLFRAME)
         dwInstFreeMemK += (GetFreeSpace(GMEM_NOT_BANKED) / 1024);

    /* Create a main window for this application instance.  */
  
    wsprintf(buffer, "MemGauge Started With %dK Free", dwInstFreeMemK);
    
    hWnd = CreateWindow(
    "MemGaugeWClass",               /* See RegisterClass() call.          */
    (LPSTR)buffer,
    WS_OVERLAPPEDWINDOW,            /* Window style.                      */
    CW_USEDEFAULT,                  /* Default horizontal position.       */
    CW_USEDEFAULT,                  /* Default vertical position.         */
    MY_WIDTH,                       /* Default width.                     */
    MY_HEIGHT,                      /* Default height.                    */
    NULL,                           /* Overlapped windows have no parent. */
    NULL,                           /* Use the window class menu.         */
    hInstance,                      /* This instance owns this window.    */
    NULL                            /* Pointer not needed.                */
    );
  
    /* If window could not be created, return "failure" */
  
    if (!hWnd)
        return (FALSE);
  
    if (!SetTimer( hWnd, TIMER_ID, 1500, NULL ))
        {
        MessageBox( hWnd, "Too many timers in use.",
                    "MemGaugeWClass", MB_ICONEXCLAMATION | MB_OK);
        return FALSE;
        }
        
    /* Make the window visible; update its client area; and return "success" */
  
    ShowWindow(hWnd, nCmdShow);  /* Show the window                        */
    UpdateWindow(hWnd);          /* Sends WM_PAINT message                 */
    return (TRUE);               /* Returns the value from PostQuitMessage */
  
    }
  
/****************************************************************************
  
    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)
  
    PURPOSE:  Processes messages
  
    MESSAGES:
  
    WM_COMMAND    - application menu (About dialog box)
    WM_DESTROY    - destroy window
  
    COMMENTS:
  
    To process the IDM_ABOUT message, call MakeProcInstance() to get the
    current instance address of the About() function.  Then call Dialog
    box which will create the box according to the information in your
    memgauge.rc file and turn control over to the About() function. When
    it returns, free the intance address.
  
****************************************************************************/
  
long FAR PASCAL MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;                    /* window handle               */
unsigned message;             /* type of message             */
WORD wParam;                  /* additional information      */
LONG lParam;                  /* additional information      */
    {
    FARPROC lpProcAbout;      /* pointer to the "About" function */
    PAINTSTRUCT ps;
    HDC         hdc;
    char outBuff[80];
    static long dwFreeMem, dwPrevMem;
    static RECT  rect;
    static int nTop, nBottom, nLeft, nRight, nRectRight;
    static int nGreen;
    HPEN  hOldPen;  

  
        switch (message) {
        case WM_CREATE:
            /* Create the pen */

            hSolidPen = CreatePen(PS_SOLID,              /* style */
                1,                                       /* width */
                RGB(0, 0, 0));                           /* color */

            break;


        case WM_TIMER:
            dwFreeMem = GetFreeSpace(0);

            if (GetWinFlags() & WF_LARGEFRAME || GetWinFlags() & WF_SMALLFRAME)
                dwFreeMem += GetFreeSpace(GMEM_NOT_BANKED);

            if (dwFreeMem != dwPrevMem)
                InvalidateRect (hWnd, NULL, TRUE);

            dwPrevMem = dwFreeMem;
            return (0);
            
        case WM_COMMAND:       /* message: command from application menu */
                if (wParam == IDM_ABOUT) {
                lpProcAbout = MakeProcInstance(About, hInst);
  
                DialogBox(hInst,         /* current instance         */
                "AboutBox",              /* resource to use          */
                hWnd,                    /* parent handle            */
                lpProcAbout);            /* About() instance address */
  
                FreeProcInstance(lpProcAbout);
                break;
                }
            else                         /* Lets Windows process it  */
                return (DefWindowProc(hWnd, message, wParam, lParam));

        case WM_SIZE:
             InvalidateRect(hWnd, NULL, TRUE);
             break;

        case WM_PAINT:
            hdc = BeginPaint (hWnd, &ps);

            GetClientRect(hWnd, &rect);

            nTop = rect.top + ((rect.bottom - rect.top) / 4);
            nBottom = rect.bottom - ((rect.bottom - rect.top) / 4);
            nLeft = rect.left + ((rect.right - rect.left) / 10);
            nRight = rect.right - ((rect.right - rect.left) / 10);


            hOldPen = SelectObject(hdc, hSolidPen);

            Rectangle(hdc, nLeft, nTop, nRight, nBottom);

            SelectObject(hdc, hOldPen);

            nGreen = ((dwFreeMem / 1024) * 255 ) / dwInstFreeMemK;
            
            hMyBrush = CreateSolidBrush(RGB(255 - nGreen, nGreen, 0));

            hOldBrush = SelectObject(hdc, hMyBrush);

            if((nRectRight = (((dwInstFreeMemK - (dwFreeMem / 1024)) * (nRight - nLeft)) / dwInstFreeMemK) + nLeft) <= nLeft )
                nRectRight = nLeft;

            Rectangle(hdc, nLeft, nTop, nRectRight, nBottom);
        
            SelectObject(hdc, hOldBrush);

            DeleteObject(hMyBrush);

            EndPaint(hWnd, &ps);
            return( 0 );

        case WM_DESTROY:          /* message: window being destroyed */
            KillTimer (hWnd, TIMER_ID);
            DeleteObject(hSolidPen);
            PostQuitMessage(0);
            break;
  
        default:                  /* Passes it on if unproccessed    */
            return (DefWindowProc(hWnd, message, wParam, lParam));
        }
    return (NULL);
    }
  
  
/****************************************************************************
  
    FUNCTION: About(HWND, unsigned, WORD, LONG)
  
    PURPOSE:  Processes messages for "About" dialog box
  
    MESSAGES:
  
    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received
  
    COMMENTS:
  
    No initialization is needed for this particular dialog box, but TRUE
    must be returned to Windows.
  
    Wait for user to click on "Ok" button, then close the dialog box.
  
****************************************************************************/
  
BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
    {
        switch (message) {
        case WM_INITDIALOG:               /* message: initialize dialog box  */
            return (TRUE);
  
        case WM_COMMAND:                      /* message: received a command */
            if (wParam == IDOK                /* "OK" box selected?          */
                || wParam == IDCANCEL) {      /* System menu close command?  */
                EndDialog(hDlg, TRUE);        /* Exits the dialog box        */
                return (TRUE);
                }
            break;
        }
    return (FALSE);                       /* Didn't process a message        */
    }
